import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:red5pro/red5_configuration.dart';

typedef void R5StreamViewCallback(R5StreamViewController controller);

enum RecordType {
  Live,
  Record,
  Append,
}

class Red5StreamView extends StatelessWidget {
  static const StandardMessageCodec _decoder = StandardMessageCodec();

  final Red5Configuration red5configuration;
  final int height;
  final int width;
  final int bitrate;
  final int fps;

  final R5StreamViewCallback? onR5StreamViewCreated;

  const Red5StreamView(
      {required this.red5configuration,
      this.height = 1080,
      this.width = 1920,
      this.bitrate = 3500,
      this.fps = 30,
      this.onR5StreamViewCreated,
      Key? key})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    final String viewType = 'Red5StreamView';
    final Map<String, dynamic> creationParams = <String, dynamic>{};
    creationParams['configuration'] = red5configuration.toJson();
    creationParams['height'] = height;
    creationParams['width'] = width;
    creationParams['bitrate'] = bitrate;
    creationParams['fps'] = fps;
    switch (defaultTargetPlatform) {
      case TargetPlatform.android:
        return AndroidView(
            viewType: viewType,
            onPlatformViewCreated: _onPlatformViewCreated,
            creationParams: creationParams,
            creationParamsCodec: _decoder);
      case TargetPlatform.iOS:
      return UiKitView(
          viewType: viewType,
          onPlatformViewCreated: _onPlatformViewCreated,
          creationParams: creationParams,
          creationParamsCodec: _decoder,
        );
      default:
        throw UnsupportedError("Unsupported platform view");
    }
  }

  void _onPlatformViewCreated(int id) {
    if (onR5StreamViewCreated == null) {
      return;
    }
    onR5StreamViewCreated!(R5StreamViewController(id));
  }
}

class R5StreamViewController {
  late MethodChannel _channel;

  R5StreamViewController(int id) {
    _channel = MethodChannel('Red5StreamView/$id');
    _channel.setMethodCallHandler(_handleMethod);
  }

  Future<dynamic> _handleMethod(MethodCall call) async {
    print(call.method);
  }

  Future<bool> switchCamera() async {
    try {
      final bool result = await _channel.invokeMethod('switchCamera');
      print("switchCamera: $result");
      return result;
    } on PlatformException catch (e) {
      print("Error switchCamera: $e.message");
    }
    return false;
  }

  Future<bool> isPublishing() async {
    try {
      final bool result = await _channel.invokeMethod('isPublishing');
      print("isPublishing: $result");
      return result;
    } on PlatformException catch (e) {
      print("Error switchCamera: $e.message");
    }
    return false;
  }

  Future<bool> startStreaming(String streamKey, RecordType recordType) async {
    try {
      final bool result = await _channel.invokeMethod('startStreaming', {
        'streamKey': streamKey,
        'recordType': _recordTypeToInt(recordType),
      });
      print("startStreaming: $result");
      return result;
    } on PlatformException catch (e) {
      print("Error startStreaming: $e.message");
    }
    return false;
  }

  Future<bool> stopStreaming() async {
    try {
      final bool result = await _channel.invokeMethod('stopStreaming');
      print("stopStreaming: $result");
      return result;
    } on PlatformException catch (e) {
      print("Error stopStreaming: $e.message");
    }
    return false;
  }

  int _recordTypeToInt(RecordType recordType) {
    switch (recordType) {
      case RecordType.Live:
        return 0;
      case RecordType.Record:
        return 1;
      case RecordType.Append:
        return 2;
      default:
        return 0;
    }
  }

  MethodChannel get channel => _channel;
}
